import json
from django.forms.widgets import Textarea
from django.forms.util import flatatt
from django.utils.safestring import mark_safe
from django.utils.html import conditional_escape
from django.utils.encoding import force_unicode
from django.core.urlresolvers import reverse
from django.conf import settings

from views import UPLOAD_PATH


IMPERAVI_SETTINGS = getattr(settings, 'IMPERAVI_CUSTOM_SETTINGS', {})


class ImperaviWidget(Textarea):

    def __init__(self, *args, **kwargs):
        self.upload_path = kwargs.pop('upload_path', UPLOAD_PATH)
        self.imperavi_settings = IMPERAVI_SETTINGS
        super(ImperaviWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        final_attrs = self.build_attrs(attrs, name=name)
        field_id = final_attrs.get('id')
        self.imperavi_settings.update({
            'imageUpload': reverse('imperavi-upload-image', kwargs={'upload_path': self.upload_path}),
            'imageGetJson': reverse('imperavi-get-json', kwargs={'upload_path': self.upload_path}),
            'fileUpload': reverse('imperavi-upload-file', kwargs={'upload_path': self.upload_path}),
            'linkFileUpload': reverse('imperavi-upload-link-file', kwargs={'upload_path': self.upload_path}),
        })
        imperavi_settings = json.dumps(self.imperavi_settings)
        return mark_safe(u"""
            <div style="width: 800px;">
                <textarea%(attrs)s>%(value)s</textarea>
            </div>
            <script>
                $(document).ready(
                    function() {
                        $("#%(id)s").parent().siblings('label').css('float', 'none');
                        $("#%(id)s").height(300);
                        $("#%(id)s").redactor(%(imperavi_settings)s);
                    }
                );
            </script>
            """ % {
                'attrs': flatatt(final_attrs),
                'value': conditional_escape(force_unicode(value)),
                'id': field_id,
                'imperavi_settings': imperavi_settings,
            }
        )



from django.contrib import admin
from django.db.models import TextField
from django.conf import settings

from widget import ImperaviWidget


class ImperaviAdmin(admin.ModelAdmin):

    unique_media = False

    formfield_overrides = {
        TextField: {'widget': ImperaviWidget},
    }

    def formfield_for_dbfield(self, db_field, **kwargs):
        field = super(ImperaviAdmin, self).formfield_for_dbfield(db_field, **kwargs)
        if getattr(field, 'widget', False) and field.widget.__class__ == ImperaviWidget and self.unique_media:
            field.widget.upload_path += '%s/' % db_field.model.__name__.lower()
        return field

    class Media:
        js = (
            '%simperavi/jquery.js' % settings.STATIC_URL,
            '%simperavi/redactor/redactor.min.js' % settings.STATIC_URL,
        )
        css = {
            'all': (
                "%simperavi/redactor/css/redactor.css" % settings.STATIC_URL,
            ),
        }


class ImperaviStackedInlineAdmin(admin.StackedInline):

    unique_media = False

    formfield_overrides = {
        TextField: {'widget': ImperaviWidget},
    }

    def formfield_for_dbfield(self, db_field, **kwargs):
        field = super(ImperaviStackedInlineAdmin, self).formfield_for_dbfield(db_field, **kwargs)
        if getattr(field, 'widget', False) and field.widget.__class__ == ImperaviWidget and self.unique_media:
            field.widget.upload_path += '%s/' % db_field.model.__name__.lower()
        return field

    class Media:
        js = (
            '%simperavi/jquery.js' % settings.STATIC_URL,
            '%simperavi/redactor/redactor.min.js' % settings.STATIC_URL,
        )
        css = {
            'all': (
                "%simperavi/redactor/css/redactor.css" % settings.STATIC_URL,
            ),
        }



from django.conf.urls import patterns, url

from views import upload_image, uploaded_images_json, upload_file

urlpatterns = patterns('',
    url(r'^upload-image/(?P<upload_path>.*)', upload_image, name="imperavi-upload-image"),
    url(r'^get-json/(?P<upload_path>.*)', uploaded_images_json, name="imperavi-get-json"),
    url(r'^upload-file/(?P<upload_path>.*)', upload_file, name="imperavi-upload-file"),
    url(r'^upload-link-file/(?P<upload_path>.*)', upload_file, {'upload_link': True}, name="imperavi-upload-link-file"),
)



import md5
import json
import os.path
import imghdr
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from django.contrib.auth.decorators import user_passes_test
from django.core.files.storage import default_storage
from django.http import HttpResponse, HttpResponseForbidden
from django.conf import settings

from forms import ImageForm, FileForm

from sorl.thumbnail import get_thumbnail


UPLOAD_PATH = getattr(settings, 'IMPERAVI_UPLOAD_PATH', 'imperavi/')


@require_POST
@csrf_exempt
@user_passes_test(lambda user: user.is_staff)
def upload_image(request, upload_path=None):
    form = ImageForm(request.POST, request.FILES)
    if form.is_valid():
        image = form.cleaned_data['file']
        if image.content_type not in ['image/png', 'image/jpg', 'image/jpeg', 'image/pjpeg']:
            return HttpResponse('Bad image format')
        image_name, extension = os.path.splitext(image.name)
        m = md5.new(image_name)
        hashed_name = '{0}{1}'.format(m.hexdigest(), extension)
        image_path = default_storage.save(os.path.join(upload_path or UPLOAD_PATH, hashed_name), image)
        image_url = default_storage.url(image_path)
        return HttpResponse(json.dumps({'filelink': image_url}))
    return HttpResponseForbidden()


@user_passes_test(lambda user: user.is_staff)
def uploaded_images_json(request, upload_path=None):
    upload_path = upload_path or UPLOAD_PATH
    results = list()
    path = os.path.join(settings.MEDIA_ROOT, upload_path)
    if os.path.isdir(path):
        for image in os.listdir(path):
            image_path = '{0}{1}'.format(path, image)
            if not os.path.isdir(image_path) and imghdr.what(image_path):
                thumb = get_thumbnail(image_path, '100x74', crop='center')
                image_url = os.path.join(settings.MEDIA_URL, upload_path, image)
                results.append({'thumb': thumb.url, 'image': image_url})
        return HttpResponse(json.dumps(results))
    return HttpResponse('{}')


@require_POST
@csrf_exempt
@user_passes_test(lambda user: user.is_staff)
def upload_file(request, upload_path=None, upload_link=None):
    form = FileForm(request.POST, request.FILES)
    if form.is_valid():
        uploaded_file = form.cleaned_data['file']
        path = os.path.join(upload_path or UPLOAD_PATH, uploaded_file.name)
        image_path = default_storage.save(path, uploaded_file)
        image_url = default_storage.url(image_path)
        if upload_link:
            return HttpResponse(image_url)
        return HttpResponse(json.dumps({'filelink': image_url, 'filename': uploaded_file.name}))
    return HttpResponseForbidden()



from setuptools import setup, find_packages

setup(
    name="django-imperavi",
    version="0.2.0",
    author="Vasyl Stanislavchuk",
    author_email="vasyl.stanislavchuk@gmail.com",
    packages=find_packages(),
    include_package_data=True,
    description="A django application that contains a class for admin interface to render a \
        text field as beautiful Imperavi WYSIWYG editor http://redactorjs.com/",
    long_description=open('README.rst', 'r').read(),
    license="MIT License",
    keywords="django admin widget imperavi",
    platforms=['any'],
    install_requires=[
        "django",
        "PIL",
        "sorl-thumbnail >= 11.12",
    ],
    url="https://github.com/vasyabigi/django-imperavi",
)
